import hashlib
import ssl
import cryptography.x509
from cryptography.hazmat.primitives.serialization import PublicFormat, Encoding

from openai import OpenAI, AsyncOpenAI
from openai.resources.chat import Chat as OpenAIChat
from openai.resources.embeddings import Embeddings as OpenAIEmbeddings
from openai.resources.audio import Audio as OpenAIAudio

from .client import SecureClient

class TinfoilAI:
    chat: OpenAIChat
    embeddings: OpenAIEmbeddings
    audio: OpenAIAudio
    api_key: str
    enclave: str

    def __init__(self, enclave: str = "inference.tinfoil.sh", repo: str = "tinfoilsh/confidential-inference-proxy", api_key: str = "tinfoil", measurement: dict = None):
        if measurement is not None:
            repo = ""
        
        # Ensure at least one verification method is provided
        if measurement is None and (repo == "" or repo is None):
            raise ValueError("Must provide either 'measurement' or 'repo' parameter for verification.")
        
        self.enclave = enclave
        self.api_key = api_key
        tf_client = SecureClient(enclave, repo, measurement)
        secure_http = tf_client.make_secure_http_client()
        self.client = OpenAI(
            base_url=f"https://{enclave}/v1/",
            api_key=api_key,
            http_client=secure_http,
        )
        self.chat = self.client.chat
        self.embeddings = self.client.embeddings
        self.audio = self.client.audio

class AsyncTinfoilAI:
    """
    Exactly like TinfoilAI, but fully async using AsyncOpenAI and httpx.AsyncClient.
    """
    chat: OpenAIChat
    embeddings: OpenAIEmbeddings
    audio: OpenAIAudio
    api_key: str
    enclave: str

    def __init__(self, enclave: str = "inference.tinfoil.sh", repo: str = "tinfoilsh/confidential-inference-proxy", api_key: str = "tinfoil", measurement: dict = None):
        if measurement is not None:
            repo = ""
        
        # Ensure at least one verification method is provided
        if measurement is None and (repo == "" or repo is None):
            raise ValueError("Must provide either 'measurement' or 'repo' parameter for verification.")
        
        self.enclave = enclave
        self.api_key = api_key
        # verifier client remains sync; only used to fetch the expected public key
        tf_client = SecureClient(enclave, repo, measurement)
        async_http = tf_client.make_secure_async_http_client()
        self.client = AsyncOpenAI(
            base_url=f"https://{enclave}/v1/",
            api_key=api_key,
            http_client=async_http,
        )
        self.chat = self.client.chat
        self.embeddings = self.client.embeddings
        self.audio = self.client.audio

class _HTTPSecureClient:
    """Low-level HTTP client with enclave-pinned TLS."""
    def __init__(self, enclave: str, tf_client, api_key: str):
        self.enclave = enclave
        self._tf_client = tf_client
        self._http_client = tf_client.make_secure_http_client()
        self._api_key = api_key

    def get(self, url: str, headers: dict = None, params: dict = None, timeout: int = None):
        return self._http_client.get(url, headers=headers, params=params, timeout=timeout)

    def post(
        self,
        url: str,
        headers: dict = None,
        data: dict = None,
        json: dict = None,
        timeout: int = None,
    ):
        return self._http_client.post(url, headers=headers, data=data, json=json, timeout=timeout)


def NewSecureClient(enclave: str = "inference.tinfoil.sh", repo: str = "tinfoilsh/confidential-inference-proxy", api_key: str = "tinfoil", measurement: dict = None):
    """
    Create a secure HTTP client for direct GET/POST through the Tinfoil enclave.
    """
    if measurement is not None:
        repo = ""
    
    # Ensure at least one verification method is provided
    if measurement is None and (repo == "" or repo is None):
        raise ValueError("Must provide either 'measurement' or 'repo' parameter for verification.")
    
    tf_client = SecureClient(enclave, repo, measurement)
    return _HTTPSecureClient(enclave, tf_client, api_key)

__all__ = ["TinfoilAI", "AsyncTinfoilAI", "NewSecureClient"]



"""
Run only when TINFOIL_* environment variables are present.
The job that sets those vars lives in .github/workflows/integration.yml.
"""

import os
import pytest
import asyncio
from pathlib import Path
from tinfoil import TinfoilAI, AsyncTinfoilAI

TEST_AUDIO_PATH = Path(__file__).parent / "jackhammer.wav"
TEST_AUDIO_TEXT = "The stale smell of old beer lingers."
ENCLAVE = "audio-processing.model.tinfoil.sh"
REPO = "tinfoilsh/confidential-audio-processing"

@pytest.fixture(scope="session")
def client() -> TinfoilAI:
    return TinfoilAI(
        enclave=ENCLAVE,
        repo=REPO,
        api_key=os.getenv("TINFOIL_API_KEY", "tinfoil"),
    )


@pytest.fixture(scope="session")
def async_client() -> AsyncTinfoilAI:
    return AsyncTinfoilAI(
        enclave=ENCLAVE,
        repo=REPO,
        api_key=os.getenv("TINFOIL_API_KEY", "tinfoil"),
    )


def test_audio_transcription(client):
    """Test synchronous audio transcription."""
    with open(TEST_AUDIO_PATH, "rb") as audio_file:
        transcription = client.audio.transcriptions.create(
            file=audio_file,
            model="whisper-large-v3-turbo",
        )
    assert transcription.text.strip() == TEST_AUDIO_TEXT
    print(f"Transcription: {transcription.text}")


@pytest.mark.asyncio
async def test_async_audio_transcription(async_client):
    """Test asynchronous audio transcription."""
    with open(TEST_AUDIO_PATH, "rb") as audio_file:
        transcription = await async_client.audio.transcriptions.create(
            file=audio_file,
            model="whisper-large-v3-turbo",
        )
    assert transcription.text.strip() == TEST_AUDIO_TEXT
    print(f"Async Transcription: {transcription.text}")



import os
import pytest
import sys

# Adjust these imports based on your project structure
from tinfoil.github import fetch_latest_digest, fetch_attestation_bundle
from tinfoil.sigstore import verify_attestation
from tinfoil.attestation import fetch_attestation

pytestmark = pytest.mark.integration  # allows pytest -m integration filtering

# Fetch config from environment variables, falling back to defaults
# Use the same env vars as the other integration test for consistency
ENCLAVE = "inference.tinfoil.sh"
REPO    = "tinfoilsh/confidential-inference-proxy"

def test_full_verification_flow():
    """
    Tests the complete attestation verification flow:
    1. Fetch latest digest for the repository.
    2. Fetch the sigstore attestation bundle for that digest.
    3. Verify the sigstore bundle to get code measurements.
    4. Fetch the runtime attestation from the enclave.
    5. Verify the runtime attestation.
    6. Compare code measurements with runtime measurements.
    """
    try:
        # Fetch latest release digest
        print(f"Fetching latest release for {REPO}")
        digest = fetch_latest_digest(REPO)
        print(f"Found digest: {digest}")

        # Fetch attestation bundle
        print(f"Fetching attestation bundle for {REPO}@{digest}")
        sigstore_bundle = fetch_attestation_bundle(REPO, digest)
        assert sigstore_bundle is not None # Basic check

        # Verify attested measurements from sigstore bundle
        print(f"Verifying attested measurements for {REPO}@{digest}")
        code_measurements = verify_attestation(
            sigstore_bundle,
            digest,
            REPO
        )
        assert code_measurements is not None # Basic check
        print(f"Code measurements fingerprint: {code_measurements.fingerprint()}")


        # Fetch runtime attestation from the enclave
        print(f"Fetching runtime attestation from {ENCLAVE}")
        enclave_attestation = fetch_attestation(ENCLAVE)
        assert enclave_attestation is not None # Basic check

        # Verify enclave measurements from runtime attestation
        print("Verifying enclave measurements")
        runtime_verification = enclave_attestation.verify()
        assert runtime_verification is not None # Basic check
        print(f"Runtime measurement fingerprint: {runtime_verification.measurement.fingerprint()}")
        print(f"Public key fingerprint: {runtime_verification.public_key_fp}")


        # Compare measurements
        print("Comparing measurements")
        assert len(code_measurements.registers) == len(runtime_verification.measurement.registers), \
            "Number of measurement registers differ"

        for i, code_reg in enumerate(code_measurements.registers):
            runtime_reg = runtime_verification.measurement.registers[i]
            assert code_reg == runtime_reg, \
                f"Measurement register {i} mismatch: Code='{code_reg}' vs Runtime='{runtime_reg}'"

        print("Verification successful!")
        print(f"Public key fingerprint: {runtime_verification.public_key_fp}")
        print(f"Measurement: {code_measurements.fingerprint()}")

    except Exception as e:
        import traceback
        traceback.print_exc()
        pytest.fail(f"Verification flow failed with exception: {e}")


if __name__ == "__main__":
    # Allow running the test directly using `python tests/test_verification_flow.py`
    pytest.main([__file__])



import pytest
from tinfoil.attestation.validate import (
    ValidationOptions, 
    validate_report, 
    _validate_policy, 
    _compare_policy_versions, 
    _validate_platform_info
)
from tinfoil.attestation.abi_sevsnp import (
    Report, 
    SnpPolicy, 
    SnpPlatformInfo, 
    TCBParts,
    SignerInfo,
    ReportSigner
)
from tinfoil.attestation.verify import CertificateChain


class TestValidationOptions:
    """Test the ValidationOptions dataclass"""
    
    def test_default_values(self):
        """Test that ValidationOptions has correct default values"""
        options = ValidationOptions()
        
        # Policy / version constraints
        assert options.guest_policy is None
        assert options.minimum_guest_svn is None
        assert options.minimum_build is None
        assert options.minimum_version is None
        
        # TCB requirements
        assert options.minimum_tcb is None
        assert options.minimum_launch_tcb is None
        assert options.permit_provisional_firmware == False
        
        # Field equality checks
        assert options.report_data is None
        assert options.host_data is None
        assert options.image_id is None
        assert options.family_id is None
        assert options.report_id is None
        assert options.report_id_ma is None
        assert options.measurement is None
        assert options.chip_id is None
        
        # Misc
        assert options.platform_info is None
        assert options.vmpl is None
        
        # ID-block / author key requirements
        assert options.require_author_key == False
        assert options.require_id_block == False

    def test_custom_values(self):
        """Test creating ValidationOptions with custom values"""
        policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=True, migrate_ma=False,
            debug=False, single_socket=True, cxl_allowed=False,
            mem_aes256_xts=True, rapl_dis=True, ciphertext_hiding_dram=True,
            page_swap_disabled=False
        )
        
        tcb = TCBParts(bl_spl=7, tee_spl=0, snp_spl=14, ucode_spl=71)
        
        platform_info = SnpPlatformInfo(
            smt_enabled=True, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=True, ciphertext_hiding_dram_enabled=True,
            alias_check_complete=True, tio_enabled=False
        )
        
        options = ValidationOptions(
            guest_policy=policy,
            minimum_guest_svn=5,
            minimum_build=21,
            minimum_version=355,  # 1.99
            minimum_tcb=tcb,
            minimum_launch_tcb=tcb,
            permit_provisional_firmware=False,
            report_data=b"A" * 64,
            host_data=b"B" * 32,
            image_id=b"C" * 16,
            family_id=b"D" * 16,
            report_id=b"E" * 32,
            report_id_ma=b"F" * 32,
            measurement=b"G" * 48,
            chip_id=b"H" * 64,
            platform_info=platform_info,
            vmpl=1,
            require_author_key=True,
            require_id_block=True
        )
        
        assert options.guest_policy == policy
        assert options.minimum_guest_svn == 5
        assert options.minimum_build == 21
        assert options.minimum_version == 355
        assert options.minimum_tcb == tcb
        assert options.minimum_launch_tcb == tcb
        assert options.permit_provisional_firmware == False
        assert options.report_data == b"A" * 64
        assert options.host_data == b"B" * 32
        assert options.image_id == b"C" * 16
        assert options.family_id == b"D" * 16
        assert options.report_id == b"E" * 32
        assert options.report_id_ma == b"F" * 32
        assert options.measurement == b"G" * 48
        assert options.chip_id == b"H" * 64
        assert options.platform_info == platform_info
        assert options.vmpl == 1
        assert options.require_author_key == True
        assert options.require_id_block == True


class TestValidatePolicy:
    """Test the _validate_policy helper function"""
    
    def test_valid_policy_match(self):
        """Test policy validation with matching policies"""
        policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=True, migrate_ma=False,
            debug=False, single_socket=True, cxl_allowed=False,
            mem_aes256_xts=True, rapl_dis=True, ciphertext_hiding_dram=True,
            page_swap_disabled=False
        )
        
        # Exact match should pass (no exception raised)
        _validate_policy(policy, policy)  # Should not raise
    
    def test_abi_version_compatibility(self):
        """Test ABI version compatibility checks"""
        report_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        
        # Required version lower than report - should pass
        required_lower = SnpPolicy(
            abi_minor=0, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        _validate_policy(report_policy, required_lower)  # Should not raise
        
        # Required version higher than report - should fail
        required_higher = SnpPolicy(
            abi_minor=2, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="Required ABI version"):
            _validate_policy(report_policy, required_higher)

    def test_unauthorized_capabilities(self):
        """Test rejection of unauthorized capabilities"""
        base_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        
        required_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        
        # Test each unauthorized capability
        # migrate_ma
        report_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=True,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="unauthorized migration agent capability"):
            _validate_policy(report_policy, required_policy)
        
        # debug
        report_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=True, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="unauthorized debug capability"):
            _validate_policy(report_policy, required_policy)
        
        # smt
        report_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=True, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="unauthorized symmetric multithreading"):
            _validate_policy(report_policy, required_policy)
        
        # cxl_allowed
        report_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=True,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="unauthorized CXL capability"):
            _validate_policy(report_policy, required_policy)

    def test_missing_required_restrictions(self):
        """Test rejection when required restrictions are missing"""
        base_report = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        
        # Test each required restriction
        # single_socket
        required_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=True, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="single socket restriction not present"):
            _validate_policy(base_report, required_policy)
        
        # mem_aes256_xts
        required_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=True, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="unauthorized memory encryption mode"):
            _validate_policy(base_report, required_policy)
        
        # rapl_dis
        required_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=True, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="unauthorized RAPL capability"):
            _validate_policy(base_report, required_policy)
        
        # ciphertext_hiding_dram
        required_policy = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=True,
            page_swap_disabled=False
        )
        with pytest.raises(ValueError, match="Ciphertext hiding in DRAM isn't enforced"):
            _validate_policy(base_report, required_policy)


class TestComparePolicyVersions:
    """Test the _compare_policy_versions helper function"""
    
    def test_equal_versions(self):
        """Test comparing equal versions"""
        policy1 = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        policy2 = SnpPolicy(
            abi_minor=1, abi_major=2, smt=True, migrate_ma=True,  # Other fields don't matter
            debug=True, single_socket=True, cxl_allowed=True,
            mem_aes256_xts=True, rapl_dis=True, ciphertext_hiding_dram=True,
            page_swap_disabled=True
        )
        
        assert _compare_policy_versions(policy1, policy2) == 0
        assert _compare_policy_versions(policy2, policy1) == 0
    
    def test_different_major_versions(self):
        """Test comparing different major versions"""
        policy_v1 = SnpPolicy(
            abi_minor=1, abi_major=1, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        policy_v2 = SnpPolicy(
            abi_minor=1, abi_major=2, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        
        assert _compare_policy_versions(policy_v1, policy_v2) < 0  # v1 < v2
        assert _compare_policy_versions(policy_v2, policy_v1) > 0  # v2 > v1
    
    def test_different_minor_versions(self):
        """Test comparing different minor versions with same major"""
        policy_v10 = SnpPolicy(
            abi_minor=0, abi_major=1, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        policy_v11 = SnpPolicy(
            abi_minor=1, abi_major=1, smt=False, migrate_ma=False,
            debug=False, single_socket=False, cxl_allowed=False,
            mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
            page_swap_disabled=False
        )
        
        assert _compare_policy_versions(policy_v10, policy_v11) < 0  # 1.0 < 1.1
        assert _compare_policy_versions(policy_v11, policy_v10) > 0  # 1.1 > 1.0


class TestValidatePlatformInfo:
    """Test the _validate_platform_info helper function"""
    
    def test_exact_match(self):
        """Test platform info validation with exact match"""
        platform_info = SnpPlatformInfo(
            smt_enabled=True, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=True, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=True, tio_enabled=False
        )
        
        _validate_platform_info(platform_info, platform_info)  # Should not raise
    
    def test_unauthorized_features(self):
        """Test rejection of unauthorized platform features"""
        base_required = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        
        # SMT enabled when not allowed
        report_info = SnpPlatformInfo(
            smt_enabled=True, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        with pytest.raises(ValueError, match="Unauthorized platform feature SMT enabled"):
            _validate_platform_info(report_info, base_required)
    
    def test_missing_required_features(self):
        """Test rejection when required platform features are missing"""
        base_report = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        
        # TSME required but not enabled
        required_info = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=True, ecc_enabled=False,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        with pytest.raises(ValueError, match="Required platform feature TSME not enabled"):
            _validate_platform_info(base_report, required_info)
        
        # RAPL disabled required but not disabled
        required_info = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=True, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        with pytest.raises(ValueError, match="Required platform feature RAPL not disabled"):
            _validate_platform_info(base_report, required_info)
        
        # Ciphertext hiding required but not enabled
        required_info = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=True,
            alias_check_complete=False, tio_enabled=False
        )
        with pytest.raises(ValueError, match="Required ciphertext hiding in DRAM not enforced"):
            _validate_platform_info(base_report, required_info)
        
        # Alias check required but not complete
        required_info = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=True, tio_enabled=False
        )
        with pytest.raises(ValueError, match="Required memory alias check hasn't been completed"):
            _validate_platform_info(base_report, required_info)

        # ECC  required but not complete
        required_info = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=False, ecc_enabled=True,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        with pytest.raises(ValueError, match="Required platform feature ECC not enabled"):
            _validate_platform_info(base_report, required_info)


def create_mock_report(**kwargs):
    """Create a mock Report object for testing"""
    # Default values
    defaults = {
        'version': 2,
        'guest_svn': 0,
        'policy': (1 << 17),  # Reserved bit set
        'family_id': b'\x00' * 16,
        'image_id': b'\x00' * 16,
        'vmpl': 0,
        'signature_algo': 1,
        'current_tcb': 0x480e000000000007,
        'platform_info': 0,
        'signer_info': 0,
        'report_data': b'\x00' * 64,
        'measurement': b'\x00' * 48,
        'host_data': b'\x00' * 32,
        'id_key_digest': b'\x00' * 48,
        'author_key_digest': b'\x00' * 48,
        'report_id': b'\x00' * 32,
        'report_id_ma': b'\x00' * 32,
        'reported_tcb': 0x480e000000000007,
        'chip_id': b'\x00' * 64,
        'committed_tcb': 0x480e000000000007,
        'current_build': 21,
        'current_minor': 55,
        'current_major': 1,
        'committed_build': 21,
        'committed_minor': 55,
        'committed_major': 1,
        'launch_tcb': 0x480e000000000007,
        'signed_data': b'\x00' * 672,
        'signature': b'\x00' * 512,
        'family': 0x19,
        'model': 0x11,
        'stepping': 0x01,
        'productName': 'Genoa'
    }
    
    # Override with provided kwargs
    defaults.update(kwargs)
    
    # Create mock object
    class MockReport:
        def __init__(self, **attrs):
            for key, value in attrs.items():
                setattr(self, key, value)
            
            # Parse helper structures
            self.policy_parsed = SnpPolicy.from_int(self.policy)
            self.platform_info_parsed = SnpPlatformInfo.from_int(self.platform_info)
            self.signer_info_parsed = SignerInfo()
            self.signer_info_parsed.signingKey = ReportSigner.VcekReportSigner
            self.signer_info_parsed.maskChipKey = False
            self.signer_info_parsed.authorKeyEn = False
    
    return MockReport(**defaults)


class MockCertificateChain:
    """Mock certificate chain for testing"""
    def validate_vcek_tcb(self, tcb):
        pass  # Do nothing for tests
    
    def validate_vcek_hwid(self, chip_id):
        pass  # Do nothing for tests


class TestValidateReport:
    """Test the main validate_report function"""
    
    def test_validate_empty_options(self):
        """Test validation with empty options (should always pass)"""
        report = create_mock_report()
        chain = MockCertificateChain()
        options = ValidationOptions()
        
        validate_report(report, chain, options)  # Should not raise
    
    def test_guest_policy_validation(self):
        """Test guest policy validation"""
        report = create_mock_report(policy=(1 << 17))  # Only reserved bit set
        chain = MockCertificateChain()
        
        # Matching policy should pass
        options = ValidationOptions(
            guest_policy=SnpPolicy.from_int((1 << 17))
        )
        validate_report(report, chain, options)  # Should not raise
        
        # Non-matching policy should fail
        options = ValidationOptions(
            guest_policy=SnpPolicy(
                abi_minor=1, abi_major=2, smt=True, migrate_ma=False,
                debug=False, single_socket=False, cxl_allowed=False,
                mem_aes256_xts=False, rapl_dis=False, ciphertext_hiding_dram=False,
                page_swap_disabled=False
            )
        )
        with pytest.raises(ValueError):
            validate_report(report, chain, options)
    
    def test_minimum_guest_svn(self):
        """Test minimum guest SVN validation"""
        chain = MockCertificateChain()
        report = create_mock_report(guest_svn=5)
        
        # Lower or equal minimum should pass
        options = ValidationOptions(minimum_guest_svn=5)
        validate_report(report, chain, options)  # Should not raise
        
        options = ValidationOptions(minimum_guest_svn=3)
        validate_report(report, chain, options)  # Should not raise
        
        # Higher minimum should fail
        options = ValidationOptions(minimum_guest_svn=10)
        with pytest.raises(ValueError, match="Guest SVN .* is less than minimum required"):
            validate_report(report, chain, options)
    
    def test_minimum_build(self):
        """Test minimum build validation"""
        chain = MockCertificateChain()
        report = create_mock_report(current_build=21, committed_build=21)
        
        # Lower or equal minimum should pass
        options = ValidationOptions(minimum_build=21)
        validate_report(report, chain, options)  # Should not raise
        
        options = ValidationOptions(minimum_build=20)
        validate_report(report, chain, options)  # Should not raise
        
        # Higher minimum should fail
        options = ValidationOptions(minimum_build=25)
        with pytest.raises(ValueError, match="SNP firmware build number .* is less than minimum required"):
            validate_report(report, chain, options)
        
        # Test when current and committed differ (both must meet minimum)
        report = create_mock_report(current_build=25, committed_build=20)
        options = ValidationOptions(minimum_build=22)
        with pytest.raises(ValueError, match="Committed SNP firmware build number .* is less than minimum required"):
            validate_report(report, chain, options)
    
    def test_minimum_version(self):
        """Test minimum version validation"""
        chain = MockCertificateChain()
        # Version 1.55 = (1 << 8) | 55 = 311
        report = create_mock_report(
            current_major=1, current_minor=55,
            committed_major=1, committed_minor=55
        )
        
        # Lower or equal minimum should pass
        options = ValidationOptions(minimum_version=311)  # 1.55
        validate_report(report, chain, options)  # Should not raise
        
        options = ValidationOptions(minimum_version=300)  # 1.44
        validate_report(report, chain, options)  # Should not raise
        
        # Higher minimum should fail
        options = ValidationOptions(minimum_version=320)  # 1.64
        with pytest.raises(ValueError, match="version .* is less than minimum required"):
            validate_report(report, chain, options)
    
    def test_minimum_tcb(self):
        """Test minimum TCB validation"""
        chain = MockCertificateChain()
        # TCB with all components at level 7
        tcb_value = (7 << 56) | (7 << 48) | (7 << 8) | 7
        report = create_mock_report(
            current_tcb=tcb_value,
            committed_tcb=tcb_value,
            reported_tcb=tcb_value
        )
        
        # Lower or equal minimum should pass
        min_tcb = TCBParts(bl_spl=7, tee_spl=7, snp_spl=7, ucode_spl=7)
        options = ValidationOptions(minimum_tcb=min_tcb)
        validate_report(report, chain, options)  # Should not raise
        
        min_tcb = TCBParts(bl_spl=5, tee_spl=5, snp_spl=5, ucode_spl=5)
        options = ValidationOptions(minimum_tcb=min_tcb)
        validate_report(report, chain, options)  # Should not raise
        
        # Higher minimum should fail
        min_tcb = TCBParts(bl_spl=10, tee_spl=10, snp_spl=10, ucode_spl=10)
        options = ValidationOptions(minimum_tcb=min_tcb)
        with pytest.raises(ValueError, match="TCB .* does not meet minimum requirements"):
            validate_report(report, chain, options)
    
    def test_field_equality_checks(self):
        """Test field equality validation"""
        chain = MockCertificateChain()
        report_data = b"A" * 64
        host_data = b"B" * 32
        measurement = b"C" * 48
        
        report = create_mock_report(
            report_data=report_data,
            host_data=host_data,
            measurement=measurement
        )
        
        # Matching fields should pass
        options = ValidationOptions(
            report_data=report_data,
            host_data=host_data,
            measurement=measurement
        )
        validate_report(report, chain, options)  # Should not raise
        
        # Non-matching fields should fail
        options = ValidationOptions(report_data=b"X" * 64)
        with pytest.raises(ValueError, match="Report data mismatch"):
            validate_report(report, chain, options)
        
        options = ValidationOptions(host_data=b"Y" * 32)
        with pytest.raises(ValueError, match="Host data mismatch"):
            validate_report(report, chain, options)
        
        options = ValidationOptions(measurement=b"Z" * 48)
        with pytest.raises(ValueError, match="Measurement mismatch"):
            validate_report(report, chain, options)
    
    def test_vmpl_validation(self):
        """Test VMPL validation"""
        chain = MockCertificateChain()
        report = create_mock_report(vmpl=1)
        
        # Matching VMPL should pass
        options = ValidationOptions(vmpl=1)
        validate_report(report, chain, options)  # Should not raise
        
        # Non-matching VMPL should fail
        options = ValidationOptions(vmpl=2)
        with pytest.raises(ValueError, match="VMPL mismatch"):
            validate_report(report, chain, options)
        
        # Invalid VMPL values should fail
        report = create_mock_report(vmpl=5)  # VMPL > 3
        options = ValidationOptions(vmpl=5)
        with pytest.raises(ValueError, match="VMPL .* is not in valid range"):
            validate_report(report, chain, options)
    
    def test_provisional_firmware_check(self):
        """Test provisional firmware validation"""
        chain = MockCertificateChain()
        # When permit_provisional_firmware=True, should always fail (not supported)
        report = create_mock_report()
        options = ValidationOptions(permit_provisional_firmware=True)
        with pytest.raises(ValueError, match="Provisional firmware is not supported"):
            validate_report(report, chain, options)
        
        # When permit_provisional_firmware=False, committed and current must match
        report = create_mock_report(
            current_build=21, committed_build=21,
            current_minor=55, committed_minor=55,
            current_major=1, committed_major=1,
            current_tcb=0x480e000000000007, committed_tcb=0x480e000000000007
        )
        options = ValidationOptions(permit_provisional_firmware=False)
        validate_report(report, chain, options)  # Should not raise
        
        # Mismatched current/committed should fail
        report = create_mock_report(
            current_build=22, committed_build=21  # Different builds
        )
        with pytest.raises(ValueError, match="Committed build .* does not match current build"):
            validate_report(report, chain, options)
        
        report = create_mock_report(
            current_minor=56, committed_minor=55  # Different minor versions
        )
        with pytest.raises(ValueError, match="Committed minor version .* does not match current minor version"):
            validate_report(report, chain, options)
    
    def test_id_block_author_key_requirements(self):
        """Test ID block and author key requirements"""
        chain = MockCertificateChain()
        report = create_mock_report()
        
        # When require_author_key=True, should fail (not supported)
        options = ValidationOptions(require_author_key=True)
        with pytest.raises(ValueError, match="ID-block and author key requirements are not supported"):
            validate_report(report, chain, options)
        
        # When require_id_block=True, should fail (not supported)
        options = ValidationOptions(require_id_block=True)
        with pytest.raises(ValueError, match="ID-block and author key requirements are not supported"):
            validate_report(report, chain, options)
    
    def test_platform_info_validation(self):
        """Test platform info validation"""
        chain = MockCertificateChain()
        platform_info_value = (1 << 0) | (1 << 3)  # SMT enabled, RAPL disabled
        report = create_mock_report(platform_info=platform_info_value)
        
        # Matching platform info should pass
        expected_info = SnpPlatformInfo(
            smt_enabled=True, tsme_enabled=False, ecc_enabled=False,
            rapl_disabled=True, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        options = ValidationOptions(platform_info=expected_info)
        validate_report(report, chain, options)  # Should not raise
        
        # Non-matching platform info should fail
        different_info = SnpPlatformInfo(
            smt_enabled=False, tsme_enabled=True, ecc_enabled=False,
            rapl_disabled=False, ciphertext_hiding_dram_enabled=False,
            alias_check_complete=False, tio_enabled=False
        )
        options = ValidationOptions(platform_info=different_info)
        with pytest.raises(ValueError):
            validate_report(report, chain, options)


if __name__ == '__main__':
    pytest.main([__file__]) 


# tests/test_integration_async.py
import os
import pytest
from tinfoil import AsyncTinfoilAI

pytestmark = pytest.mark.integration

ENCLAVE = "inference.tinfoil.sh"
REPO    = "tinfoilsh/confidential-inference-proxy"
API_KEY = os.getenv("TINFOIL_API_KEY", "tinfoil")

@pytest.mark.asyncio
async def test_async_chat_integration():
    client = AsyncTinfoilAI(
        enclave=ENCLAVE,
        repo=REPO,
        api_key=API_KEY,
    )
    # perform a streaming chat completion
    stream = await client.chat.completions.create(
        model="llama3-3-70b",
        messages=[{"role": "user", "content": "Hello from async integration"}],
        stream=True,
    )
    collected = []
    async for chunk in stream:
        if chunk.choices[0].delta.content is not None:
            collected.append(chunk.choices[0].delta.content)
    output = "".join(collected)
    assert output, "Expected non-empty response from async streaming API"



import os
import pytest
from tinfoil import SecureClient

@pytest.fixture(scope="session")
def client() -> SecureClient:
    return SecureClient(
        enclave="doc-upload.model.tinfoil.sh",
        repo="tinfoilsh/confidential-doc-upload",
    )

def test_doc_upload(client):
    """Test synchronous doc upload."""
    httpx_client = client.make_secure_http_client()

    with open("tests/dummy.pdf", "rb") as file:
        response = httpx_client.post(
            "https://doc-upload.model.tinfoil.sh/v1/convert/file", 
            files={'files': file},
            timeout=30,
        )
        assert response.status_code == 200
        assert response.json()["status"] == "success"

@pytest.mark.asyncio
async def test_doc_upload_async(client):
    """Test asynchronous doc upload."""
    httpx_client = client.make_secure_async_http_client()

    with open("tests/dummy.pdf", "rb") as file:
        response = await httpx_client.post(
            "https://doc-upload.model.tinfoil.sh/v1/convert/file", 
            files={'files': file},
            timeout=30,
        )
        assert response.status_code == 200
        assert response.json()["status"] == "success"



from dataclasses import dataclass
from enum import IntEnum

POLICY_RESERVED_1_BIT = 17
REPORT_SIZE = 0x4A0  # 1184 bytes
SIGNATURE_OFFSET = 0x2A0
ECDSA_RS_SIZE = 72
ECDSA_P384_SHA384_SIGNATURE_SIZE = ECDSA_RS_SIZE + ECDSA_RS_SIZE

ZEN3ZEN4_FAMILY = 0x19
ZEN5_FAMILY     = 0x1A
MILAN_MODEL     = 0 | 1
GENOA_MODEL     = (1 << 4) | 1
TURIN_MODEL     = 2

class ReportSigner(IntEnum):
    VcekReportSigner = 0
	# VlekReportSigner is the SIGNING_KEY value for if the VLEK signed the attestation report.
    VlekReportSigner = 1
    endorseReserved2 = 2
    endorseReserved3 = 3
    endorseReserved4 = 4
    endorseReserved5 = 5
    endorseReserved6 = 6
    # NoneReportSigner is the SIGNING_KEY value for if the attestation report is not signed.
    NoneReportSigner = 7
        
# SignerInfo represents information about the signing circumstances for the attestation report.
class SignerInfo:
	# SigningKey represents kind of key by which a report was signed.
	signingKey: ReportSigner
	# MaskChipKey is true if the host chose to enable CHIP_ID masking, to cause the report's CHIP_ID
	# to be all zeros.
	maskChipKey: bool
	# AuthorKeyEn is true if the VM is launched with an IDBLOCK that includes an author key.
	authorKeyEn: bool

@dataclass
class TCBParts:
    """Represents the decomposed parts of a TCB version"""
    ucode_spl: int
    snp_spl: int
    tee_spl: int
    bl_spl: int

    def __str__(self) -> str:
        """Return a human-friendly string with all component SPL values."""
        # Print fields in order starting with the least-significant component (bl_spl)
        return (
            "TCBParts("  # Opening
            f"bl_spl=0x{self.bl_spl:02x}, "
            f"tee_spl=0x{self.tee_spl:02x}, "
            f"snp_spl=0x{self.snp_spl:02x}, "
            f"ucode_spl=0x{self.ucode_spl:02x})"
        )

    @classmethod
    def from_int(cls, tcb: int) -> "TCBParts":
        """Build a TCBParts instance from a 64-bit packed TCB value."""
        return cls(
            ucode_spl=((tcb >> 56) & 0xff),
            snp_spl=((tcb >> 48) & 0xff),
            tee_spl=((tcb >> 8) & 0xff),
            bl_spl=((tcb >> 0) & 0xff),
        )

    def meets_minimum(self, minimum: "TCBParts") -> bool:
        """Check if this TCB meets minimum requirements (component-wise)."""
        return (
            self.bl_spl >= minimum.bl_spl and
            self.tee_spl >= minimum.tee_spl and
            self.snp_spl >= minimum.snp_spl and
            self.ucode_spl >= minimum.ucode_spl
        )

@dataclass
class SnpPlatformInfo:
    """Decoded view of the 64-bit PLATFORM_INFO field."""

    smt_enabled: bool
    tsme_enabled: bool
    ecc_enabled: bool
    rapl_disabled: bool
    ciphertext_hiding_dram_enabled: bool
    alias_check_complete: bool
    tio_enabled: bool

    @classmethod
    def from_int(cls, value: int) -> "SnpPlatformInfo":
        return cls(
            smt_enabled=bool(value & (1 << 0)),
            tsme_enabled=bool(value & (1 << 1)),
            ecc_enabled=bool(value & (1 << 2)),
            rapl_disabled=bool(value & (1 << 3)),
            ciphertext_hiding_dram_enabled=bool(value & (1 << 4)),
            alias_check_complete=bool(value & (1 << 5)),
            tio_enabled=bool(value & (1 << 7))
        )

    def __str__(self) -> str:  # pragma: no cover – formatting helper
        return (
            "SnpPlatformInfo("  # opening
            f"SMTEnabled={self.smt_enabled}, "
            f"TSMEEnabled={self.tsme_enabled}, "
            f"ECCEnabled={self.ecc_enabled}, "
            f"RAPLDisabled={self.rapl_disabled}, "
            f"CiphertextHidingDRAMEnabled={self.ciphertext_hiding_dram_enabled}, "
            f"AliasCheckComplete={self.alias_check_complete}, "
            f"TIOEnabled={self.tio_enabled})"
        )


@dataclass
class SnpPolicy:
    """Decoded view of the 64-bit POLICY field (bits 0-20)."""

    abi_minor: int
    abi_major: int
    smt: bool
    migrate_ma: bool
    debug: bool
    single_socket: bool
    cxl_allowed: bool
    mem_aes256_xts: bool
    rapl_dis: bool
    ciphertext_hiding_dram: bool
    page_swap_disabled: bool

    def __str__(self) -> str:  # pragma: no cover – formatting helper
        return (
            "SnpPolicy("  # opening
            f"ABIMajor={self.abi_major}, ABIMinor={self.abi_minor}, "
            f"SMT={self.smt}, MigrateMA={self.migrate_ma}, Debug={self.debug}, "
            f"SingleSocket={self.single_socket}, CXLAllowed={self.cxl_allowed}, "
            f"MemAES256XTS={self.mem_aes256_xts}, RAPLDis={self.rapl_dis}, "
            f"CipherTextHidingDRAM={self.ciphertext_hiding_dram}, PageSwapDisabled={self.page_swap_disabled})"
        )

    @classmethod
    def from_int(cls, value: int) -> "SnpPolicy":
        """Parse the guest policy bit-field following AMD SEV-SNP spec."""
        return cls(
            abi_minor=value & 0xFF,
            abi_major=(value >> 8) & 0xFF,
            smt=bool(value & (1 << 16)),
            migrate_ma=bool(value & (1 << 18)),
            debug=bool(value & (1 << 19)),
            single_socket=bool(value & (1 << 20)),
            cxl_allowed=bool(value & (1 << 21)),
            mem_aes256_xts=bool(value & (1 << 22)),
            rapl_dis=bool(value & (1 << 23)),
            ciphertext_hiding_dram=bool(value & (1 << 24)),
            page_swap_disabled=bool(value & (1 << 25)),
        )

@dataclass 
class Report:
    """SEV-SNP attestation report"""
    version: int  # Should be 2 for revision 1.55, 3 for revision 1.56, 5 for revision 1.58
    guest_svn: int
    policy: int
    policy_parsed: SnpPolicy
    family_id: bytes  # Should be 16 bytes long
    image_id: bytes   # Should be 16 bytes long
    vmpl: int
    signature_algo: int
    current_tcb: int
    platform_info: int
    platform_info_parsed: SnpPlatformInfo
    signer_info: int  # AuthorKeyEn, MaskChipKey, SigningKey
    signer_info_parsed: SignerInfo
    report_data: bytes  # Should be 64 bytes long
    measurement: bytes  # Should be 48 bytes long
    host_data: bytes   # Should be 32 bytes long
    id_key_digest: bytes  # Should be 48 bytes long
    author_key_digest: bytes  # Should be 48 bytes long
    report_id: bytes   # Should be 32 bytes long
    report_id_ma: bytes  # Should be 32 bytes long
    reported_tcb: int
    chip_id: bytes  # Should be 64 bytes long
    committed_tcb: int
    current_build: int
    current_minor: int
    current_major: int
    committed_build: int
    committed_minor: int
    committed_major: int
    launch_tcb: int
    signed_data: bytes
    signature: bytes  # Should be 512 bytes long
    family: bytes
    model: bytes
    stepping: bytes
    productName: str

    def __init__(self, data: bytes):
        """
        Parse an attestation report from raw bytes in SEV SNP ABI format.
        
        Args:
            data: Raw bytes of the attestation report
        Returns:
            Report object containing parsed data
        """

        if len(data) < REPORT_SIZE:
            raise ValueError(f"Array size is 0x{len(data):x}, an SEV-SNP attestation report size is 0x{REPORT_SIZE:x}")

        # Parse all fields using little-endian byte order
        self.version = int.from_bytes(data[0x00:0x04], byteorder='little')
        self.guest_svn = int.from_bytes(data[0x04:0x08], byteorder='little')
        self.policy = int.from_bytes(data[0x08:0x10], byteorder='little')
        
        # Check reserved bit must be 1
        if not (self.policy & (1 << POLICY_RESERVED_1_BIT)):
            raise ValueError(f"policy[{POLICY_RESERVED_1_BIT}] is reserved, must be 1, got 0")
        
        # Check bits 63-26 must be zero
        if self.policy >> 26:
            raise ValueError("policy bits 63-26 must be zero")
    
        self.family_id = data[0x10:0x20]  # 16 bytes
        self.image_id = data[0x20:0x30]   # 16 bytes
        self.vmpl = int.from_bytes(data[0x30:0x34], byteorder='little')
        self.signature_algo = int.from_bytes(data[0x34:0x38], byteorder='little')
        self.current_tcb = int.from_bytes(data[0x38:0x40], byteorder='little')

        try:
            mbz64(int(self.current_tcb), "current_tcb", 47, 16)
        except ValueError as e:
            raise ValueError(f"current_tcb not correctly formed: {e}")
        
        self.platform_info = int.from_bytes(data[0x40:0x48], byteorder='little')
        # Decode additional helper structures for easier consumption later.
        self.policy_parsed = SnpPolicy.from_int(self.policy)
        self.platform_info_parsed = SnpPlatformInfo.from_int(self.platform_info)
        self.signer_info = int.from_bytes(data[0x48:0x4C], byteorder='little')

        self.signer_info_parsed = SignerInfo()
        try:
            mbz64(int(self.signer_info), "signer_info", 31, 5)
        except ValueError as e:
            raise ValueError(f"signer_info not correctly formed: {e}")
        
        self.signer_info_parsed.signingKey = ReportSigner((self.signer_info >> 2) & 7)
        if self.signer_info_parsed.signingKey != ReportSigner.VcekReportSigner:
            raise ValueError(f"This implementation only supports VCEK signed reports. Got {self.signer_info_parsed.signingKey}")
        self.signer_info_parsed.maskChipKey = (self.signer_info & 2) != 0
        self.signer_info_parsed.authorKeyEn = (self.signer_info & 1) != 0

        try:
            mbz(data, 0x4C, 0x50)
        except ValueError as e:
            raise ValueError(f"report_data not correctly formed: {e}")

        # 0x4C-0x50 is MBZ (Must Be Zero)
        self.report_data = data[0x50:0x90]      # 64 bytes
        self.measurement = data[0x90:0xC0]       # 48 bytes
        self.host_data = data[0xC0:0xE0]        # 32 bytes
        self.id_key_digest = data[0xE0:0x110]    # 48 bytes
        self.author_key_digest = data[0x110:0x140]  # 48 bytes
        self.report_id = data[0x140:0x160]       # 32 bytes
        self.report_id_ma = data[0x160:0x180]    # 32 bytes
        self.reported_tcb = int.from_bytes(data[0x180:0x188], byteorder='little')

        try:
            mbz64(int(self.reported_tcb), "reported_tcb", 47, 16)
        except ValueError as e:
            raise ValueError(f"reported_tcb not correctly formed: {e}")

        mbzLo = 0x188
        # Version specific parsing
        if self.version >= 3:  # Report Version 3
            self.family = data[0x188]
            self.model = data[0x189]
            self.stepping = data[0x18A]
            self._init_product_name()
            mbzLo = 0x18B
        elif self.version == 2:  # Report Version 2
            self.family = ZEN3ZEN4_FAMILY
            self.model = GENOA_MODEL
            self.stepping = 0x01
            self.productName = "Genoa"
        else:
            raise ValueError("Unknown report version")
        
        try:
            mbz(data, mbzLo, 0x1A0)
        except ValueError as e:
            raise ValueError(f"report_data not correctly formed: {e}")
        
        self.chip_id = data[0x1A0:0x1E0]        # 64 bytes
        self.committed_tcb = int.from_bytes(data[0x1E0:0x1E8], byteorder='little')

        try:
            mbz64(int(self.committed_tcb), "committed_tcb", 47, 16)
        except ValueError as e:
            raise ValueError(f"committed_tcb not correctly formed: {e}")
        
        # Version fields
        self.current_build = data[0x1E8]
        self.current_minor = data[0x1E9]
        self.current_major = data[0x1EA]

        try:
            mbz(data, 0x1EB, 0x1EC)
        except ValueError as e:
            raise ValueError(f"report_data not correctly formed: {e}")
        
        self.committed_build = data[0x1EC]
        self.committed_minor = data[0x1ED]
        self.committed_major = data[0x1EE]
        
        try:
            mbz(data, 0x1EF, 0x1F0)
        except ValueError as e:
            raise ValueError(f"report_data not correctly formed: {e}")
        
        self.launch_tcb = int.from_bytes(data[0x1F0:0x1F8], byteorder='little')

        try:
            mbz64(int(self.launch_tcb), "launch_tcb", 47, 16)
        except ValueError as e:
            raise ValueError(f"launch_tcb not correctly formed: {e}")

        try:
            mbz(data, 0x1F8, SIGNATURE_OFFSET)
        except ValueError as e:
            raise ValueError(f"report_data not correctly formed: {e}")
        
        if self.signature_algo == 1: # ECDSA P-384 SHA-384
            try:
                mbz(data, SIGNATURE_OFFSET+ECDSA_P384_SHA384_SIGNATURE_SIZE, REPORT_SIZE)
            except ValueError as e:
                raise ValueError(f"report_data not correctly formed: {e}")
        
        self.signed_data = data[0:SIGNATURE_OFFSET]
        self.signature = data[SIGNATURE_OFFSET:REPORT_SIZE]
        
    def get_fms(self):
        return self.family, self.model, self.stepping
    
    def _init_product_name(self):
        # Combined extended values
        self.productName = "Unknown"
        if self.family == ZEN3ZEN4_FAMILY:
            if self.model == MILAN_MODEL:
                self.productName = "Milan"
            elif self.model == GENOA_MODEL:
                self.productName = "Genoa"
        elif self.family == ZEN5_FAMILY:
            if self.model == TURIN_MODEL:
                self.productName = "Turin"

    def print_report(self):
        """Print all relevant fields of the SEV-SNP attestation report in a human-readable format."""
        print("=== SEV-SNP Attestation Report ===")
        print(f"Version: {self.version}")
        print(f"Guest SVN: {self.guest_svn}")
        print(f"Policy: 0x{self.policy:x}")
        print(f"  -> {self.policy_parsed}")
        print(f"Family ID: {self.family_id.hex()}")
        print(f"Image ID: {self.image_id.hex()}")
        print(f"VMPL: {self.vmpl}")
        print(f"Signature Algorithm: {self.signature_algo}")
        print(f"Current TCB: 0x{self.current_tcb:x}")
        print(f"  -> {TCBParts.from_int(self.current_tcb)}")
        print(f"Platform Info: 0x{self.platform_info:x}")
        print(f"  -> {self.platform_info_parsed}")
        print(f"Signer Info: 0x{self.signer_info:x}")
        print(f"  - Signing Key: {self.signer_info_parsed.signingKey}")
        print(f"  - Mask Chip Key: {self.signer_info_parsed.maskChipKey}")
        print(f"  - Author Key Enabled: {self.signer_info_parsed.authorKeyEn}")
        print(f"Report Data: {self.report_data.hex()}")
        print(f"Measurement: {self.measurement.hex()}")
        print(f"Host Data: {self.host_data.hex()}")
        print(f"ID Key Digest: {self.id_key_digest.hex()}")
        print(f"Author Key Digest: {self.author_key_digest.hex()}")
        print(f"Report ID: {self.report_id.hex()}")
        print(f"Report ID MA: {self.report_id_ma.hex()}")
        print(f"Reported TCB: 0x{self.reported_tcb:x}")
        print(f"  -> {TCBParts.from_int(self.reported_tcb)}")
        print(f"Chip ID: {self.chip_id.hex()}")
        print(f"Committed TCB: 0x{self.committed_tcb:x}")
        print(f"  -> {TCBParts.from_int(self.committed_tcb)}")
        print(f"Current Version: {self.current_major}.{self.current_minor}.{self.current_build}")
        print(f"Committed Version: {self.committed_major}.{self.committed_minor}.{self.committed_build}")
        print(f"Launch TCB: 0x{self.launch_tcb:x}")
        print(f"  -> {TCBParts.from_int(self.launch_tcb)}")
        print(f"Product Name: {self.productName}")
        if hasattr(self, 'family') and hasattr(self, 'model') and hasattr(self, 'stepping'):
            print(f"CPU: Family=0x{self.family:02x}, Model=0x{self.model:02x}, Stepping=0x{self.stepping:02x}")
        print(f"Signature Length: {len(self.signature)} bytes")
        print("=" * 40)

## HELPER FUNCTIONS
    
def find_non_zero(data: bytes, lo: int, hi: int) -> int:
    """
    Returns the first index which is not zero, otherwise returns hi.
    
    Args:
        data: Bytes object to search through
        lo: Starting index (inclusive)
        hi: Ending index (exclusive)
    Returns:
        Index of first non-zero byte, or hi if all bytes are zero
    """
    for i in range(lo, hi):
        if data[i] != 0:
            return i
    return hi

def mbz(data: bytes, lo: int, hi: int) -> None:
    """
    Checks if a range of bytes is all zeros.
    
    Args:
        data: Bytes object to check
        lo: Starting index (inclusive)
        hi: Ending index (exclusive)
    Raises:
        ValueError: If any byte in the range is non-zero
    """
    first_non_zero = find_non_zero(data, lo, hi)
    if first_non_zero != hi:
        # Convert the slice to hex string for error message
        hex_str = data[lo:hi].hex()
        raise ValueError(f"mbz range [0x{lo:x}:0x{hi:x}] not all zero: {hex_str}")

def mbz64(data: int, base: str, hi: int, lo: int) -> None:
    """
    Checks if a range of bits in an integer is all zeros.
    
    Args:
        data: Integer to check
        base: String identifier for error message
        hi: Highest bit position (inclusive)
        lo: Lowest bit position (inclusive)
    Raises:
        ValueError: If any bit in the range is non-zero
    """
    # Create mask for the bit range
    mask = (1 << (hi - lo + 1)) - 1
    # Extract and check the bits
    bits = (data >> lo) & mask
    if bits != 0:
        raise ValueError(f"mbz range {base}[0x{lo:x}:0x{hi:x}] not all zero: {hex(data)}")
    
